//
//  ServerConnection.m
//  LocationApp_Sample
//
//  Copyright (c) 2022 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "ServerConnection.h"

@interface ServerConnection()

/* 通信 */
- (void)_connect:(NSMutableURLRequest *)request response:(NSHTTPURLResponse **)response data:(NSDictionary **)data error:(NSError **)error;

@end


@implementation ServerConnection

#pragma mark Public

/* トークン発行リクエスト */
- (void)requestGetToken:(NSString *)url clientId:(NSString *)clientId secret:(NSString *)secret response:(NSHTTPURLResponse **)response data:(NSDictionary **)data error:(NSError **)error
{
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc]init];

    [request setURL:[NSURL URLWithString:url]];
    [request setHTTPMethod:@"POST"];
    [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
    NSString *baseString = [NSString stringWithFormat:@"%@:%@", clientId, secret];;
    NSData *encodedString = [baseString dataUsingEncoding:NSUTF8StringEncoding];
    NSString *base64String = [NSString stringWithFormat:@"Basic %@",[encodedString base64EncodedStringWithOptions:0]];
    [request setValue:base64String forHTTPHeaderField:@"Authorization"];
    [request setHTTPBody:[@"grant_type=client_credentials" dataUsingEncoding:NSUTF8StringEncoding]];

    [self _connect:request response:response data:data error:error];
}

/* 位置蓄積サーバリクエスト */
- (void)requestUploadLogs:(NSString *)url tokenId:(NSString *)tokenId apikey:(NSString *)apikey json:(NSString *)jsonString response:(NSHTTPURLResponse **)response data:(NSDictionary **)data error:(NSError **)error
{
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc]init];

    [request setURL:[NSURL URLWithString:url]];
    [request setHTTPMethod:@"POST"];
    [request setValue:@"application/x-www-form-urlencoded; charset=UTF-8" forHTTPHeaderField:@"Content-Type"];
    NSString *auth = [NSString stringWithFormat:@"Bearer %@", tokenId];;
    [request setValue:auth forHTTPHeaderField:@"authorization"];
    [request setValue:apikey forHTTPHeaderField:@"x-api-key"];
    NSString *body = [NSString stringWithFormat:@"request_data=%@", jsonString];
    [request setHTTPBody:[body dataUsingEncoding:NSUTF8StringEncoding]];

    [self _connect:request response:response data:data error:error];
}

/* レスポンスコード成功判定 */
- (BOOL)isResponseOK:(NSHTTPURLResponse *)response
{
    return (response.statusCode >= 200 && response.statusCode < 300);
}

#pragma mark Private

/* 通信 */

- (void)_connect:(NSMutableURLRequest *)request response:(NSHTTPURLResponse **)response data:(NSDictionary **)data error:(NSError **)error
{
    dispatch_semaphore_t semaphore = dispatch_semaphore_create(0);

    __block NSMutableDictionary *json = nil;
    __block NSHTTPURLResponse *httpUrlResponse = nil;
    __block NSError *errorInfo = nil;

    NSURLSessionConfiguration *config = [NSURLSessionConfiguration defaultSessionConfiguration];
    config.timeoutIntervalForRequest = 60;
    config.timeoutIntervalForResource = 60;

    NSURLSession *session = [NSURLSession sessionWithConfiguration: config
                                                          delegate: nil
                                                     delegateQueue: nil];

    NSURLSessionDataTask *task = [session dataTaskWithRequest:request
                                            completionHandler:^(NSData *data, NSURLResponse *response, NSError *error) {
        httpUrlResponse = (NSHTTPURLResponse *)response;
        if (data != nil) {
            json = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingAllowFragments error:&error];
        }
        errorInfo = error;

        dispatch_semaphore_signal(semaphore);
    }];

    // 通信開始
    [task resume];

    // 通信完了まで待機
    dispatch_semaphore_wait(semaphore, DISPATCH_TIME_FOREVER);

    // セッションクリア
    [session invalidateAndCancel];
    session = nil;

    // 通信結果格納
    *response = httpUrlResponse;
    *data = json;
    *error = errorInfo;
}

@end
